/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.model.contact;

import java.io.Serializable;

import javax.persistence.AssociationOverride;
import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import de.fzk.iai.ilcd.service.model.IContactVO;
import de.iai.ilcd.model.common.DataSet;
import de.iai.ilcd.model.common.MultiLanguageString;

/**
 * 
 * @author clemens.duepmeier
 */
@Entity
@Table(uniqueConstraints = @UniqueConstraint(columnNames = { "UUID", "MAJORVERSION", "MINORVERSION", "SUBMINORVERSION" }))
public class Contact extends DataSet implements Serializable, IContactVO {

	@Embedded
	@AttributeOverride(name = "defaultValue", column = @Column(name = "shortName"))
	@AssociationOverride(name = "lStringMap", joinTable = @JoinTable(name = "contact_shortname", joinColumns = @JoinColumn(name = "contact_id", referencedColumnName = "ID")))
	protected MultiLanguageString shortName = new MultiLanguageString();

	protected String contactAddress = null;

	protected String phone = null;

	protected String fax = null;

	protected String email = null;

	protected String www = null;

	protected String centralContactPoint = null;

	@Override
	public String getCentralContactPoint() {
		return centralContactPoint;
	}

	public void setCentralContactPoint(String centralContactPoint) {
                if (centralContactPoint != null && centralContactPoint.length() > 256)
                    centralContactPoint=centralContactPoint.substring(0, 255);
		this.centralContactPoint = centralContactPoint;
	}

	public String getContactAddress() {
		return contactAddress;
	}

	public void setContactAddress(String contactAddress) {
		this.contactAddress = contactAddress;
	}

	@Override
	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	@Override
	public String getFax() {
		return fax;
	}

	public void setFax(String fax) {
		this.fax = fax;
	}

	@Override
	public String getWww() {
		return www;
	}

	public void setWww(String homePage) {
		this.www = homePage;
	}

	@Override
	public String getPhone() {
		return phone;
	}

	public void setPhone(String phone) {
		this.phone = phone;
	}

	@Override
	public MultiLanguageString getShortName() {
		return shortName;
	}

	public void setShortName(MultiLanguageString shortName) {
		this.shortName = shortName;
	}

	@Override
	public String toString() {
		return "de.iai.ilcd.model.contact.Contact[id=" + id + "]";
	}

}
