/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.model.lciamethod;

import javax.persistence.AssociationOverride;
import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Embeddable;
import javax.persistence.Embedded;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;

import de.fzk.iai.ilcd.service.model.common.IMultiLangString;
import de.fzk.iai.ilcd.service.model.lciamethod.ITimeInformation;
import de.iai.ilcd.model.common.MultiLanguageString;
import java.io.Serializable;

/**
 * Time information for {@link LCIAMethod LCIA methods}
 */
@Embeddable
public class TimeInformation implements ITimeInformation, Serializable {

	/**
	 * Duration information
	 */
	@Embedded
	@AttributeOverride(name = "defaultValue", column = @Column(name = "durationDescription"))
	@AssociationOverride(name = "lStringMap", joinTable = @JoinTable(name = "lciamethod_durationDescription", joinColumns = @JoinColumn(name = "lciamethod_id", referencedColumnName = "ID")))
	private MultiLanguageString duration = new MultiLanguageString();

	/**
	 * Reference year information
	 */
	@Embedded
	@AttributeOverride(name = "defaultValue", column = @Column(name = "referenceYearDescription"))
	@AssociationOverride(name = "lStringMap", joinTable = @JoinTable(name = "lciamethod_referenceYearDescription", joinColumns = @JoinColumn(name = "lciamethod_id", referencedColumnName = "ID")))
	private MultiLanguageString referenceYear = new MultiLanguageString();

	/**
	 * Get the duration information
	 * 
	 * @return duration information
	 */
	@Override
	public IMultiLangString getDuration() {
		return this.duration;
	}

	/**
	 * Get the reference year information
	 * 
	 * @return reference year information
	 */
	@Override
	public IMultiLangString getReferenceYear() {
		return this.referenceYear;
	}

	/**
	 * Set the duration information
	 * 
	 * @param duration
	 *            the new duration information
	 */
	public void setDuration(MultiLanguageString duration) {
		this.duration = duration;
	}

	/**
	 * Set the reference year information
	 * 
	 * @param referenceYear
	 *            the new reference year information
	 */
	public void setReferenceYear(MultiLanguageString referenceYear) {
		this.referenceYear = referenceYear;
	}

}
