/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.security;

import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.AuthenticationInfo;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.crypto.hash.Sha512Hash;
import org.apache.shiro.subject.PrincipalCollection;

public class IlcdSecurityRealm extends org.apache.shiro.realm.AuthorizingRealm {

	/**
	 * Name of the realm
	 */
	public static final String REALM_NAME = "dbRealm";

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection pc) {

		AuthorizationInfo authInfo = new IlcdAuthorizationInfo((String) pc.getPrimaryPrincipal());
		return authInfo;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken at) throws AuthenticationException {
		AuthenticationInfo authInfo = new IlcdAuthenticationInfo(at);
		return authInfo;
	}


	/**
	 * Clear the cache
	 * @param principals principal collection to clear cache for
	 */
	public void clearAuthorizationInfo(PrincipalCollection principals) {
		this.clearCachedAuthorizationInfo(principals);
	}
	
	/**
	 * Get the encrypted password for plain text and hash salt
	 * @param plainPassword plain text password
	 * @param hashSalt salt for the hash
	 * @return result hex string
	 */
	public static String getEncryptedPassword( String plainPassword, String hashSalt ){
		return new Sha512Hash(plainPassword, hashSalt, 5).toHex();		
	}
	
}
