/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.webgui.controller.admin;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.apache.velocity.tools.generic.ValueParser;
import org.primefaces.component.datatable.DataTable;
import org.primefaces.model.LazyDataModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.iai.ilcd.model.dao.DeleteDataSetException;
import de.iai.ilcd.model.dao.FlowDao;
import de.iai.ilcd.model.flow.Flow;

/**
 * 
 * @author clemens.duepmeier
 */
@ManagedBean()
@ViewScoped
public class ProductFlowListHandler implements Serializable {

	private static Logger logger = LoggerFactory.getLogger(ProductFlowListHandler.class);

	private LazyDataModel<Flow> lazyModel;

	private Flow[] selectedFlows;

	private ValueParser params;

	public ProductFlowListHandler() {
		lazyModel = new FlowDataModel();

		FlowDao dao = new FlowDao();
		params = new ValueParser();
		long rowCount = dao.searchResultCount(params);
		lazyModel.setRowCount((int) rowCount);
	}

	public LazyDataModel<Flow> getLazyModel() {
		return lazyModel;
	}

	public void setLazyModel(LazyDataModel<Flow> lazyModel) {
		this.lazyModel = lazyModel;
	}

	public Flow[] getSelectedFlows() {
		return selectedFlows;
	}

	public void setSelectedFlows(Flow[] selectedFlows) {
		this.selectedFlows = selectedFlows;
	}

	public void deleteSelected() {
		if (selectedFlows == null) {
			return;
		}

		FlowDao fDao = new FlowDao();

		for (Flow flow : selectedFlows) {
			try {
				fDao.remove(flow);
			} catch (DeleteDataSetException ex) {
				FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR, "Could not remove flow "
						+ flow.getName().getDefaultValue() + "\n", "");
				FacesContext.getCurrentInstance().addMessage("error", facesMsg);
			}
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, "Removed product "
					+ flow.getName().getDefaultValue() + "\n", "");
			FacesContext.getCurrentInstance().addMessage("info", facesMsg);
		}

		selectedFlows = null;

		int rowCount = (int) fDao.searchResultCount(params);
		lazyModel.setRowCount(rowCount);
		DataTable dataTable = (DataTable) FacesContext.getCurrentInstance().getViewRoot()
				.findComponent("tableForm:flowTable");
		if (dataTable != null)
			dataTable.loadLazyData();

		return;
	}

	private class FlowDataModel extends LazyDataModel<Flow> {
		// we have to overload the load()-method of the lazy data model

		@Override
		public List<Flow> load(int first, int pageSize, String sortField, boolean sortOrder, Map<String, String> filters) {
			logger.info("Loading the lazy flow data between {0} and {1}", new Object[] { first, (first + pageSize) });

			// Sorting and Filtering information are not used for demo purposes just random dummy data is returned

			List<Flow> lazyFlows = new ArrayList<Flow>();
			// now let's fill it from the database
			FlowDao dao = new FlowDao();
			// ValueParser params = new ValueParser();
			lazyFlows = dao.search(Flow.class, params, first, pageSize, sortField);

			long rowCount = dao.searchResultCount(params);
			this.setRowCount((int) rowCount);
			return lazyFlows;
		}
	}
}
