/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/


package de.iai.ilcd.webgui.controller.admin;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import de.iai.ilcd.model.dao.DataStockDao;
import de.iai.ilcd.model.dao.DeleteDataSetException;
import de.iai.ilcd.model.datastock.DataStock;

/**
 * 
 * @author clemens.duepmeier
 */
@ManagedBean
@ViewScoped
public class StockListHandler implements Serializable {

	private List<DataStock> stocks = new ArrayList<DataStock>();

	private DataStock[] selected = null;

	private DataStockDao dao = new DataStockDao();

	public StockListHandler() {
	}

	public List<DataStock> getStocks() {
		stocks = dao.getDataStocks();
		return stocks;
	}

	public DataStock[] getSelected() {
		return selected;
	}

	public void setSelected(DataStock[] selected) {
		this.selected = selected;
	}

	public void deleteSelected() {

		if (selected == null) {
			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_WARN,
					"No data stocks selected; select some first\n", "");
			FacesContext.getCurrentInstance().addMessage("warnMessage", facesMsg);
			return;
		}

		for (DataStock stock : selected) {
			try {
				dao.remove(stock);
			} catch (DeleteDataSetException ex) {
				FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR,
						"Unknown data base error: Cannot remove " + stock.getName().getDefaultValue() + "\n", "");
				FacesContext.getCurrentInstance().addMessage("errorMessage", facesMsg);
			}

			FacesMessage facesMsg = new FacesMessage(FacesMessage.SEVERITY_INFO, "Removed "
					+ stock.getName().getDefaultValue() + "\n", "");
			FacesContext.getCurrentInstance().addMessage("info", facesMsg);
		}

		selected = null;
	}
}
