/*******************************************************************************
 * Copyright (c) 2011 Karlsruhe Institute of Technology (KIT) - Institute for 
 * Applied Computer Science (IAI). 
 * 
 * This file is part of soda4LCA - the Service-Oriented Life Cycle Data Store.
 * 
 * soda4LCA is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the 
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * soda4LCA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with soda4LCA.  If not, see <http://www.gnu.org/licenses/>.
 ******************************************************************************/

package edu.kit.iai.soda4lca;

import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map.Entry;
import java.util.Random;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import org.apache.shiro.crypto.hash.Sha512Hash;

/**
 * soda4LCA Password Encryption Tool (1.1beta to 1.1final)
 */
public class PasswordEncrypter extends JFrame {
	
	/**
	 * Serialization ID
	 */
	private static final long serialVersionUID = 3532302941855599958L;
	
	/**
	 * Flag to indicate if JDBC driver was loaded successfully
	 */
	private final static boolean MYSQL_JDBC_DRIVER_LOADED = PasswordEncrypter.loadMySQLJDBCDriver();
	
	/**
	 * Charset for salt
	 */
	private static final String SALT_CHARSET = "!0123456789abcdefghijklmnopqrstuvwxyz";
	
	/**
	 * Text field for database host
	 */
	private final JTextField txtDBHost = new JTextField("localhost");

	/**
	 * Text field for database name
	 */
	private final JTextField txtDBName = new JTextField("soda4lca");
	
	/**
	 * Text field for database user name
	 */
	private final JTextField txtDBUser = new JTextField("root");

	/**
	 * Password field for password of database user
	 */
	private final JPasswordField txtDBPass = new JPasswordField();
	
	/**
	 * Create the encrypter
	 */
	public PasswordEncrypter() {
		super("soda4LCA 1.1beta => 1.1 Password Encrypter");
		if ( !PasswordEncrypter.MYSQL_JDBC_DRIVER_LOADED ){
			JOptionPane.showMessageDialog(null, "Could not load JDBC driver for MySQL, exiting", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		this.initUI();
	}

	/**
	 * Initialize user interface
	 */
	private void initUI() {
		this.setDefaultCloseOperation( WindowConstants.DISPOSE_ON_CLOSE );
		this.setSize(300, 340);
		this.setLocationRelativeTo(null);
	
		this.setLayout( new GridBagLayout() );
		
		int row = 0;
		
		
		this.addInput(this.txtDBHost, "Database Host", row++);
		this.addInput(this.txtDBName, "Database Name", row++);
		this.addInput(this.txtDBUser, "Database User", row++);
		this.addInput(this.txtDBPass, "Database Password", row++);
		
		// add button
		this.addToGridBagLayout(new JLabel("Please backup your database first!", JLabel.CENTER), row++, false);
		JPanel panBtn = new JPanel( new FlowLayout(FlowLayout.CENTER) );
		panBtn.add( new JButton( new EncryptionAction() ) );
		this.addToGridBagLayout(panBtn, row++, false);
		// fill panel
		this.addToGridBagLayout(new JPanel(), row++, true);
	}
	
	/**
	 * Add input to UI
	 * @param input input to add
	 * @param title title for border
	 * @param row row in {@link GridBagLayout}
	 */
	private void addInput( JTextField input, String title, int row ){
		JPanel pan = new JPanel( new GridLayout(1, 1) );
		pan.add( input );
		pan.setBorder( BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), " " + title + " ") );
		this.addToGridBagLayout(pan, row, false);
	}
	
	/**
	 * Add to {@link GridBagLayout}
	 * @param c component to add
	 * @param row row in layout
	 * @param fillY flag if fill shall be applied for y
	 */
	private void addToGridBagLayout( JComponent c, int row, boolean fillY ){
		this.add( c, new GridBagConstraints(0, row, 1, 1, 1.d, fillY?1.d:0.d, GridBagConstraints.BASELINE, fillY?GridBagConstraints.BOTH:GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
	}
	
	/**
	 * Action for the encryption of the plain text passwords
	 */
	public class EncryptionAction extends AbstractAction {

		/**
		 * Serialization ID
		 */
		private static final long serialVersionUID = -7449417234117849274L;

		/**
		 * Create the action
		 */
		public EncryptionAction() {
			super("Encrypt Passwords");
		}
		
		/**
		 * Add the new columns if required &amp; encrypt passwords
		 */
		@Override
		public void actionPerformed(ActionEvent e) {
			try {
				final String dbName = PasswordEncrypter.this.txtDBName.getText();
				Connection c = DriverManager.getConnection("jdbc:mysql://" + PasswordEncrypter.this.txtDBHost.getText() + "/" + dbName + "?characterEncoding=UTF8&amp;useUnicode=yes" , PasswordEncrypter.this.txtDBUser.getText(), new String(PasswordEncrypter.this.txtDBPass.getPassword()));

				// check if plain text password column exists
				if ( !this.checkUserTableColumnExists(c, dbName, "PASSWORD") ){
					throw new IllegalStateException("This database no longer contains plain text passwords - cannot convert data");
				}
				
				// create the table columns if required
				if ( !this.checkUserTableColumnExists(c, dbName, "PASSWORD_HASH") ){
					c.createStatement().execute("ALTER TABLE `user` ADD `PASSWORD_HASH` varchar(128) DEFAULT NULL");
				}

				if ( !this.checkUserTableColumnExists(c, dbName, "PASSWORD_HASH_SALT") ){
					c.createStatement().execute("ALTER TABLE `user` ADD `PASSWORD_HASH_SALT` varchar(20) DEFAULT NULL");
				}
				
				HashMap<Long, String[]> valueMap = new HashMap<Long, String[]>();
				ResultSet rs = c.createStatement().executeQuery("SELECT `ID`,`PASSWORD` FROM `user`");
				while ( rs.next() ){
					final String plainPassword = rs.getString("PASSWORD");
					final long id = rs.getLong("ID");
					final String hashSalt = this.generateSalt();
					final String passwordHash = new Sha512Hash(plainPassword, hashSalt, 5).toHex();	
					valueMap.put(new Long(id), new String[]{passwordHash,hashSalt});
				}
				
				boolean allOk = true;
				PreparedStatement pst = c.prepareStatement("UPDATE `user` SET `PASSWORD_HASH`=?, `PASSWORD_HASH_SALT`=? WHERE `ID`=?");
				for( Entry<Long, String[]> entry : valueMap.entrySet() ){
					final String[] tmp = entry.getValue();
					pst.setString(1, tmp[0]);
					pst.setString(2, tmp[1]);
					pst.setLong(3, entry.getKey().longValue());
					allOk &= pst.execute();
				}
				
				if ( allOk ){
					c.createStatement().execute("ALTER TABLE `user` DROP `PASSWORD`");
				}
				
				JOptionPane.showMessageDialog(PasswordEncrypter.this, "Password encryption finished", "Done", JOptionPane.INFORMATION_MESSAGE);
				
			} catch (Exception ex) {
				ex.printStackTrace();
				JOptionPane.showMessageDialog(PasswordEncrypter.this, ex.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			}
		}
		
		/**
		 * Check if a specific column exists for the user table
		 * @param c connection to use
		 * @param dbName name of the database
		 * @param columnName name of the column
		 * @return <code>true</code> if column exists, else <code>false</code>
		 * @throws SQLException on database errors
		 */
		private boolean checkUserTableColumnExists(Connection c, String dbName, String columnName) throws SQLException {
			// check if plain text password column exists
			PreparedStatement pst = c.prepareStatement("SELECT count(*) FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = ? AND TABLE_NAME = 'user' AND COLUMN_NAME = ?");
			pst.setString(1, dbName);
			pst.setString(2, columnName);
			ResultSet rs = pst.executeQuery();
			boolean colExists = false;
			if ( rs.next() ){
				colExists = rs.getInt(1) == 1;
			}
			rs.close();
			return colExists;
		}
		
		/**
		 * Generate a random salt
		 * @return random salt (2 characters)
		 */
		private String generateSalt() {
			Random rand = new Random(System.currentTimeMillis());
			StringBuilder sb = new StringBuilder();
			for (int i = 0; i < 20; i++) {
				int pos = rand.nextInt(PasswordEncrypter.SALT_CHARSET.length());
				sb.append(PasswordEncrypter.SALT_CHARSET.charAt(pos));
			}
			return sb.toString();
		}
	}


	/**
	 * Start the application
	 * @param args shell arguments, ignored
	 */
	public static void main(String[] args) {
		PasswordEncrypter pe = new PasswordEncrypter();
		pe.setVisible(true);
	}
	
	/**
	 * Load JDBC driver for MySQL
	 * @return <code>true</code> if loaded successfully, else <code>false</code>
	 */
	private static boolean loadMySQLJDBCDriver() {
		try {
			Class.forName("com.mysql.jdbc.Driver");
			return true;
		} catch (ClassNotFoundException e) {
			return false;
		}
	}

	
}
